@@ -1,21 +1,22 @@
 from __future__ import annotations
-import numpy as np
 import math
 
+import numpy as np
+
 pi = math.pi
+tau = math.tau
 
 
-def geomag2geog(thetat: np.ndarray, phit: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
-    """ geomagnetic to geographic """
+def geomag2geog(thetat, phit) -> tuple:
+    """geomagnetic to geographic"""
 
     # FIXME: this is for year 1985, see Schmidt spherical harmonic in MatGemini
     thetan = math.radians(11)
     phin = math.radians(289)
 
     # enforce phit = [0,2pi]
-    phit = np.remainder(phit, 2 * pi)
+    phit = phit % tau
 
-    # thetag2p=acos(cos(thetat).*cos(thetan)-sin(thetat).*sin(thetan).*cos(phit));
     thetag2p = np.arccos(
         np.cos(thetat) * np.cos(thetan) - np.sin(thetat) * np.sin(thetan) * np.cos(phit)
     )
@@ -23,20 +24,18 @@ def geomag2geog(thetat: np.ndarray, phit: np.ndarray) -> tuple[np.ndarray, np.nd
     beta = np.arccos(
         (np.cos(thetat) - np.cos(thetag2p) * np.cos(thetan)) / (np.sin(thetag2p) * np.sin(thetan))
     )
-
-    phig2 = np.zeros_like(phit)
+    phig2 = np.empty_like(phit, dtype=float)
 
     i = phit > pi
     phig2[i] = phin - beta[i]
-
-    i = phit <= pi
+    i = np.logical_not(i)
     phig2[i] = phin + beta[i]
 
     i = phig2 < 0
-    phig2[i] = phig2[i] + 2 * pi
+    phig2[i] = phig2[i] + tau
 
-    i = phig2 >= 2 * pi
-    phig2[i] = phig2[i] - 2 * pi
+    i = phig2 >= tau
+    phig2[i] = phig2[i] - tau
 
     thetag2 = pi / 2 - thetag2p
     lat = np.degrees(thetag2)
@@ -45,40 +44,36 @@ def geomag2geog(thetat: np.ndarray, phit: np.ndarray) -> tuple[np.ndarray, np.nd
     return lat, lon
 
 
-def geog2geomag(lat: np.ndarray, lon: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
-    """ geographic to geomagnetic """
+def geog2geomag(lat, lon) -> tuple:
+    """geographic to geomagnetic"""
 
     # FIXME: this is for year 1985, see Schmidt spherical harmonic in MatGemini
     thetan = math.radians(11)
     phin = math.radians(289)
 
-    lat = np.atleast_1d(lat)
-    lon = np.atleast_1d(lon)
-
-    # enforce [0,360] longitude
-    lon = lon % 360
-
     thetagp = pi / 2 - np.radians(lat)
-    phig = np.radians(lon)
+    phig = np.radians(lon % 360)
 
     thetat = np.arccos(
         np.cos(thetagp) * np.cos(thetan) + np.sin(thetagp) * np.sin(thetan) * np.cos(phig - phin)
     )
     argtmp = (np.cos(thetagp) - np.cos(thetat) * np.cos(thetan)) / (np.sin(thetat) * np.sin(thetan))
     alpha = np.arccos(max(min(argtmp, 1), -1))
-    phit = np.empty(lat.shape)
+    phit = np.empty_like(lat, dtype=float)
 
     i = ((phin > phig) & ((phin - phig) > pi)) | ((phin < phig) & ((phig - phin) < pi))
+
     phit[i] = pi - alpha[i]
-    phit[~i] = alpha[~i] + pi
+    i = np.logical_not(i)
+    phit[i] = alpha[i] + pi
 
-    return thetat, phit
+    return thetat, phit[()]
 
 
-def geog2UEN(alt, glon, glat, thetactr, phictr):
+def geog2UEN(alt, glon, glat, thetactr, phictr) -> tuple:
     """
     Converts a set of glon,glat into magnetic up, north, east coordinates.
-    thetactr and phictr are the magnetic coordinates of the center of hte region of interest.
+    thetactr and phictr are the magnetic coordinates of the center of the region of interest.
     They can be computed from geog2geomag.
     """
 
@@ -100,7 +95,7 @@ def geog2UEN(alt, glon, glat, thetactr, phictr):
     return z, x, y
 
 
-def UEN2geog(z, x, y, thetactr, phictr):
+def UEN2geog(z, x, y, thetactr, phictr) -> tuple:
     """
     converts magnetic up, north, east coordinates into geographic coordinates.
     """
@@ -119,6 +114,6 @@ def UEN2geog(z, x, y, thetactr, phictr):
     phi = phictr + gamma1
 
     # Now convert the magnetic to geographic using our simple transformation
-    [glat, glon] = geomag2geog(theta, phi)
+    glat, glon = geomag2geog(theta, phi)
 
     return alt, glon, glat
