@@ -3,22 +3,20 @@ HDF5 file read
 """
 
 from __future__ import annotations
-import xarray
 from pathlib import Path
 import typing as T
-import numpy as np
 import logging
 from datetime import datetime, timedelta
 
+import xarray
+import numpy as np
+import h5py
+
+from gemini3d.utils import filename2datetime
+
 from .. import find
 from .. import WAVELEN
 
-try:
-    import h5py
-except (ImportError, AttributeError):
-    # must be ImportError not ModuleNotFoundError for botched HDF5 linkage
-    h5py = None
-
 
 def simsize(path: Path) -> tuple[int, ...]:
     """
@@ -35,40 +33,33 @@ def simsize(path: Path) -> tuple[int, ...]:
         3 integers telling simulation grid size
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
-    path = find.simsize(path, ".h5")
+    path = find.simsize(path)
 
     with h5py.File(path, "r") as f:
         if "lxs" in f:
-            lxs = f["lxs"][:]
+            lx = f["lxs"][:]
         elif "lx" in f:
-            lxs = f["lx"][:]
+            lx = f["lx"][:]
         elif "lx1" in f:
             if f["lx1"].ndim > 0:
-                lxs = np.array(
-                    [
+                lx = np.array(
+                    (
                         f["lx1"][:].squeeze()[()],
                         f["lx2"][:].squeeze()[()],
                         f["lx3"][:].squeeze()[()],
-                    ]
+                    )
                 )
             else:
-                lxs = np.array([f["lx1"][()], f["lx2"][()], f["lx3"][()]])
+                lx = np.array((f["lx1"][()], f["lx2"][()], f["lx3"][()]))
         else:
             raise KeyError(f"could not find '/lxs', '/lx' or '/lx1' in {path.as_posix()}")
 
-    return lxs
+    return lx
 
 
 def flagoutput(file: Path, cfg: dict[str, T.Any]) -> int:
-    """ detect output type """
+    """detect output type"""
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
-    flag = None
     with h5py.File(file, "r") as f:
         if "nsall" in f:
             # milestone or full
@@ -76,20 +67,16 @@ def flagoutput(file: Path, cfg: dict[str, T.Any]) -> int:
         elif "flagoutput" in f:
             flag = f["/flagoutput"][()]
         elif "ne" in f and f["/ne"].ndim == 3:
-            flag = 0
-        elif "Tavgall" in f:
-            flag = 2
-        elif "neall" in f:
             flag = 3
-    if flag is None:
-        flag = cfg.get("flagoutput")
+        elif "neall" in f:
+            flag = 2
+        else:
+            flag = cfg["flagoutput"]
 
     return flag
 
 
-def grid(
-    file: Path, *, var: tuple[str, ...] | list[str] = None, shape: bool = False
-) -> dict[str, T.Any]:
+def grid(file: Path, *, var: set[str] | None = None, shape: bool = False) -> dict[str, T.Any]:
     """
     get simulation grid
 
@@ -97,7 +84,7 @@ def grid(
     ----------
     file: pathlib.Path
         filepath to simgrid
-    var: list of str, optional
+    var: set of str, optional
         read only these grid variables
     shape: bool, optional
         read only the shape of the grid instead of the data iteslf
@@ -110,18 +97,10 @@ def grid(
     Transpose on read to undo the transpose operation we had to do in write_grid C => Fortran order.
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
-    xg: dict[str, T.Any] = {}
-
     if not file.is_file():
-        file2 = find.grid(file)
-        if file2 and file2.is_file():
-            file = file2
-        else:
-            logging.error(f"{file} grid file is not present.")
-            return xg
+        file = find.grid(file)
+
+    xg: dict[str, T.Any] = {"filename": file}
 
     if shape:
         with h5py.File(file, "r") as f:
@@ -131,19 +110,28 @@ def grid(
                 else:
                     xg[k] = f[k].shape
 
-        xg["lxs"] = np.array([xg["x1"], xg["x2"], xg["x3"]])
+        xg["lx"] = np.array((xg["x1"], xg["x2"], xg["x3"]))
         return xg
 
+    if isinstance(var, str):
+        var = [var]
+
     with h5py.File(file, "r") as f:
-        if not var:
-            var = f.keys()
+        var = set(var) if var else f.keys()
+
         for k in var:
             if f[k].ndim >= 2:
                 xg[k] = f[k][:].transpose()
             else:
-                xg[k] = f[k][:]
+                if f[k].size > 1:
+                    xg[k] = f[k][:]
+                else:
+                    xg[k] = f[k]
 
-    xg["lxs"] = simsize(file.with_name("simsize.h5"))
+    if file.stem == "amrgrid":
+        xg["lx"] = np.array((xg["x1"].size, xg["x2"].size, xg["x3"].size))
+    else:
+        xg["lx"] = simsize(file.parent)
 
     return xg
 
@@ -153,19 +141,16 @@ def Efield(file: Path) -> xarray.Dataset:
     load electric field
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
     with h5py.File(file.with_name("simgrid.h5"), "r") as f:
         E = xarray.Dataset(coords={"mlon": f["/mlon"][:], "mlat": f["/mlat"][:]})
 
     with h5py.File(file, "r") as f:
         E["flagdirich"] = f["flagdirich"][()].item()
-        for p in ("Exit", "Eyit", "Vminx1it", "Vmaxx1it"):
+        for p in {"Exit", "Eyit", "Vminx1it", "Vmaxx1it"}:
             E[p] = (("mlat", "mlon"), f[p][:])
-        for p in ("Vminx2ist", "Vmaxx2ist"):
+        for p in {"Vminx2ist", "Vmaxx2ist"}:
             E[p] = (("mlat",), f[p][:])
-        for p in ("Vminx3ist", "Vmaxx3ist"):
+        for p in {"Vminx3ist", "Vmaxx3ist"}:
             E[p] = (("mlon",), f[p][:])
 
     return E
@@ -176,36 +161,27 @@ def precip(file: Path) -> xarray.Dataset:
     load precipitation
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
     with h5py.File(file.with_name("simgrid.h5"), "r") as f:
         dat = xarray.Dataset(coords={"mlon": f["/mlon"][:], "mlat": f["/mlat"][:]})
 
     with h5py.File(file, "r") as f:
-        for k in ("Q", "E0"):
+        for k in {"Q", "E0"}:
             dat[k] = (("mlat", "mlon"), f[f"/{k}p"][:])
 
     return dat
 
 
-def frame3d_curvne(file: Path) -> xarray.Dataset:
+def frame3d_curvne(file: Path, xg: dict[str, T.Any] | None = None) -> xarray.Dataset:
     """
     just Ne
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
+    if not xg:
+        xg = grid(file.parent, var={"x1", "x2", "x3"})
 
-    xg = grid(file.parent, var=("x1", "x2", "x3"))
     dat = xarray.Dataset(coords={"x1": xg["x1"][2:-2], "x2": xg["x2"][2:-2], "x3": xg["x3"][2:-2]})
 
-    lxs = simsize(file.parent)
-
-    if lxs[2] == 1:  # east-west
-        p3 = (2, 0, 1)
-    else:  # 3D or north-south, no swap
-        p3 = (2, 1, 0)
+    p3 = (2, 1, 0)
 
     with h5py.File(file, "r") as f:
         dat["ne"] = (("x1", "x2", "x3"), f["/ne"][:].transpose(p3))
@@ -213,7 +189,7 @@ def frame3d_curvne(file: Path) -> xarray.Dataset:
     return dat
 
 
-def frame3d_curv(file: Path, var: tuple[str, ...] | list[str]) -> xarray.Dataset:
+def frame3d_curv(file: Path, var: set[str], xg: dict[str, T.Any] | None = None) -> xarray.Dataset:
     """
     curvilinear
 
@@ -222,90 +198,80 @@ def frame3d_curv(file: Path, var: tuple[str, ...] | list[str]) -> xarray.Dataset
 
     file: pathlib.Path
         filename to read
-    var: list of str
+    var: set of str
         variable(s) to read
     """
 
-    xg = grid(file.parent, var=("x1", "x2", "x3"))
-    dat = xarray.Dataset(coords={"x1": xg["x1"][2:-2], "x2": xg["x2"][2:-2], "x3": xg["x3"][2:-2]})
+    if isinstance(var, str):
+        var = [var]
+    var = set(var)
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
+    if not xg:
+        xg = grid(file.parent, var={"x1", "x2", "x3"})
 
-    lxs = simsize(file.parent)
+    if xg.get("filename", Path()).stem == "amrgrid":
+        # FIXME: perhaps make a config.nml flag indicating AMR grid used?
+        dat = xarray.Dataset(coords={"x1": xg["x1"], "x2": xg["x2"], "x3": xg["x3"]})
+    else:
+        dat = xarray.Dataset(
+            coords={"x1": xg["x1"][2:-2], "x2": xg["x2"][2:-2], "x3": xg["x3"][2:-2]}
+        )
 
-    p4s = (0, 3, 1, 2)
-    p3s = (2, 0, 1)
-    p4n = (0, 3, 2, 1)
-    p3n = (2, 1, 0)
+    lx = xg["lx"]
 
-    if lxs[1] == 1 or file.name == "initial_conditions.h5":
-        p4 = p4n
-        p3 = p3n
-    elif lxs[2] == 1:  # east-west
-        p4 = p4s
-        p3 = p3s
-    else:  # 3D
-        p4 = p4n
-        p3 = p3n
+    p4 = (0, 3, 2, 1)
+    p3 = (2, 1, 0)
 
     with h5py.File(file, "r") as f:
-        if {"ne", "ns", "v1", "Ti"}.intersection(var):
+
+        if {"ne", "ns", "v1", "Ti"} & var:
             dat["ns"] = (("species", "x1", "x2", "x3"), f["/nsall"][:].transpose(p4))
 
-        if {"v1", "vs1"}.intersection(var):
+        if {"v1", "vs1"} & var:
             dat["vs1"] = (("species", "x1", "x2", "x3"), f["/vs1all"][:].transpose(p4))
 
-        if {"Te", "Ti", "Ts"}.intersection(var):
+        if {"Te", "Ti", "Ts"} & var:
             dat["Ts"] = (("species", "x1", "x2", "x3"), f["/Tsall"][:].transpose(p4))
 
-        for k in {"J1", "J2", "J3"}.intersection(var):
+        for k in {"J1", "J2", "J3"} & var:
             dat[k] = (("x1", "x2", "x3"), f[f"/{k}all"][:].transpose(p3))
 
-        for k in {"v2", "v3"}.intersection(var):
+        for k in {"v2", "v3"} & var:
             dat[k] = (("x1", "x2", "x3"), f[f"/{k}avgall"][:].transpose(p3))
 
         if "Phi" in var:
             Phiall = f["/Phiall"][:]
 
             if Phiall.ndim == 1:
-                if lxs[1] == 1:
+                if lx[1] == 1:
                     Phiall = Phiall[None, :]
                 else:
                     Phiall = Phiall[:, None]
 
-            if (lxs[1] == 1 and not file.name == "initial_conditions.h5") or (
-                lxs[1] != 1 and lxs[2] != 1
-            ):
-                Phiall = Phiall.transpose()
-            dat["Phitop"] = (("x2", "x3"), Phiall)
+            dat["Phitop"] = (("x2", "x3"), Phiall.transpose())
 
     return dat
 
 
-def frame3d_curvavg(file: Path, var: tuple[str, ...] | list[str]) -> xarray.Dataset:
+def frame3d_curvavg(
+    file: Path, var: set[str], xg: dict[str, T.Any] | None = None
+) -> xarray.Dataset:
     """
 
     Parameters
     ----------
     file: pathlib.Path
         filename of this timestep of simulation output
-    var: list of str
+    var: set of str
         variable(s) to read
     """
 
-    xg = grid(file.parent, var=("x1", "x2", "x3"))
+    if not xg:
+        xg = grid(file.parent, var={"x1", "x2", "x3"})
+
     dat = xarray.Dataset(coords={"x1": xg["x1"][2:-2], "x2": xg["x2"][2:-2], "x3": xg["x3"][2:-2]})
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
-    lxs = simsize(file.parent)
-
-    if lxs[2] == 1:  # east-west
-        p3 = (2, 0, 1)
-    else:  # 3D or north-south, no swap
-        p3 = (2, 1, 0)
+    p3 = (2, 1, 0)
 
     v2n = {
         "ne": "neall",
@@ -320,7 +286,12 @@ def frame3d_curvavg(file: Path, var: tuple[str, ...] | list[str]) -> xarray.Data
         "Phi": "Phiall",
     }
 
+    if isinstance(var, str):
+        var = [var]
+
     with h5py.File(file, "r") as f:
+        var = set(var) if var else f.keys()
+
         for k in var:
             if k == "Phi":
                 dat["Phitop"] = (("x2", "x3"), f[f"/{v2n[k]}"][:].transpose())
@@ -330,7 +301,7 @@ def frame3d_curvavg(file: Path, var: tuple[str, ...] | list[str]) -> xarray.Data
     return dat
 
 
-def glow_aurmap(file: Path) -> xarray.Dataset:
+def glow_aurmap(file: Path, xg: dict[str, T.Any] | None = None) -> xarray.Dataset:
     """
     read the auroral output from GLOW
 
@@ -340,18 +311,12 @@ def glow_aurmap(file: Path) -> xarray.Dataset:
         filename of this timestep of simulation output
     """
 
-    xg = grid(file.parents[1], var=("x2", "x3"))
+    if not xg:
+        xg = grid(file.parents[1], var={"x2", "x3"})
+
     dat = xarray.Dataset(coords={"wavelength": WAVELEN, "x2": xg["x2"][2:-2], "x3": xg["x3"][2:-2]})
 
-    lxs = simsize(file.parents[1])
-
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
-
-    if lxs[2] == 1:  # east-west
-        p3 = (0, 2, 1)
-    else:  # 3D or north-south, no swap
-        p3 = (0, 2, 1)
+    p3 = (0, 2, 1)
 
     with h5py.File(file, "r") as h:
         dat["rayleighs"] = (("wavelength", "x2", "x3"), h["/aurora/iverout"][:].transpose(p3))
@@ -364,19 +329,18 @@ def time(file: Path) -> datetime:
     reads simulation time
     """
 
-    if h5py is None:
-        raise ImportError("h5py missing or broken")
+    try:
+        with h5py.File(file, "r") as f:
+            ymd = datetime(*f["/time/ymd"][:3])
 
-    with h5py.File(file, "r") as f:
-        ymd = datetime(*f["/time/ymd"][:3])
+            try:
+                hour = f["/time/UThour"][()].item()
+            except KeyError:
+                hour = f["/time/UTsec"][()].item() / 3600
 
-        if "/time/UThour" in f:
-            hour = f["/time/UThour"][()].item()
-        elif "/time/UTsec" in f:
-            hour = f["/time/UTsec"][()].item() / 3600
-        else:
-            raise KeyError(f"did not find time of day in {file}")
-
-    t = ymd + timedelta(hours=hour)
+        t = ymd + timedelta(hours=hour)
+    except KeyError:
+        logging.error(f"/time group missing from {file}, getting time from filename pattern.")
+        t = filename2datetime(file)
 
     return t
