@@ -3,21 +3,18 @@ HDF5 file writing
 """
 
 from __future__ import annotations
-import xarray
 import typing as T
-import numpy as np
 from pathlib import Path
 from datetime import datetime
 import logging
 
-from .. import LSP
-from ..utils import datetime2ymd_hourdec, to_datetime
+import h5py
+import numpy as np
+import xarray
 
-try:
-    import h5py
-except (ImportError, AttributeError):
-    # must be ImportError not ModuleNotFoundError for botched HDF5 linkage
-    h5py = None
+from ..utils import datetime2stem, to_datetime
+
+CLVL = 3  # GZIP compression level: larger => better compression, slower to write
 
 
 def state(fn: Path, dat: xarray.Dataset):
@@ -32,18 +29,10 @@ def state(fn: Path, dat: xarray.Dataset):
     I.E. THEY SHOULD NOT INCLUDE GHOST CELLS
     """
 
-    if h5py is None:
-        raise ImportError("pip install h5py")
-
-    time = dat.time
-
-    logging.info(f"state: {fn} {time}")
+    logging.info(f"state: {fn}")
 
     with h5py.File(fn, "w") as f:
-        f["/time/ymd"] = [time.year, time.month, time.day]
-        f["/time/UTsec"] = np.float32(
-            time.hour * 3600 + time.minute * 60 + time.second + time.microsecond / 1e6
-        )
+        write_time(f, to_datetime(dat.time))
 
         for k in {"ns", "vs1", "Ts"}:
             if k in dat.data_vars:
@@ -53,7 +42,7 @@ def state(fn: Path, dat: xarray.Dataset):
             _write_var(f, "/Phiall", dat["Phitop"])
 
 
-def _write_var(f, name: str, A: xarray.DataArray):
+def _write_var(fid, name: str, A: xarray.DataArray):
     """
     NOTE: The .transpose() reverses the dimension order.
     The HDF Group never implemented the intended H5T_array_create(..., perm)
@@ -65,11 +54,15 @@ def _write_var(f, name: str, A: xarray.DataArray):
     """
 
     p4s = ("species", "x3", "x2", "x1")
+    p3s = ("x3", "x2", "x1")
+    p2s = ("x3", "x2")
 
     if A.ndim == 4:
         A = A.transpose(*p4s)
+    elif A.ndim == 3:
+        A = A.transpose(*p3s)
     elif A.ndim == 2:
-        A = A.transpose()
+        A = A.transpose(*p2s)
     elif A.ndim == 1:
         pass
     else:
@@ -77,62 +70,17 @@ def _write_var(f, name: str, A: xarray.DataArray):
             f"write_hdf5: unexpected number of dimensions {A.ndim}. Please raise a GitHub Issue."
         )
 
-    f.create_dataset(
+    fid.create_dataset(
         name,
         data=A,
-        dtype=np.float32,
-        compression="gzip",
-        compression_opts=1,
+        dtype=np.float32,  # float32 saves disk space
+        compression="gzip",  # GZIP is universally available with HDF5
+        compression_opts=CLVL,
         shuffle=True,
         fletcher32=True,
     )
 
 
-def data(outfn: Path, dat: xarray.Dataset):
-    """
-    write simulation data
-    e.g. for converting a file format from a simulation
-    """
-
-    if h5py is None:
-        raise ImportError("pip install h5py")
-
-    lxs = dat.shape
-
-    with h5py.File(outfn, "w") as h:
-        for k in {"ns", "vs1", "Ts"}:
-            if k not in dat:
-                continue
-
-            h.create_dataset(
-                k,
-                data=dat[k].astype(np.float32),
-                chunks=(1, *lxs[1:], LSP),
-                compression="gzip",
-                compression_opts=1,
-            )
-
-        for k in {"ne", "v1", "Ti", "Te", "J1", "J2", "J3", "v2", "v3"}:
-            if k not in dat:
-                continue
-
-            h.create_dataset(
-                k,
-                data=dat[k].astype(np.float32),
-                chunks=(1, *lxs[1:]),
-                compression="gzip",
-                compression_opts=1,
-            )
-
-        if "Phitop" in dat:
-            h.create_dataset(
-                "Phiall",
-                data=dat["Phitop"].astype(np.float32),
-                compression="gzip",
-                compression_opts=1,
-            )
-
-
 def grid(size_fn: Path, grid_fn: Path, xg: dict[str, T.Any]):
     """writes grid to disk
 
@@ -154,20 +102,17 @@ def grid(size_fn: Path, grid_fn: Path, xg: dict[str, T.Any]):
     need the .transpose() for h5py
     """
 
-    if h5py is None:
-        raise ImportError("pip install h5py")
-
     if "lx" not in xg:
-        xg["lx"] = np.array([xg["x1"].shape, xg["x2"].shape, xg["x3"].shape])
+        xg["lx"] = np.array((xg["x1"].shape, xg["x2"].shape, xg["x3"].shape)).astype(np.int32)
 
     logging.info(f"write_grid: {size_fn}")
     with h5py.File(size_fn, "w") as h:
-        h["/lx"] = xg["lx"]
+        h["/lx"] = np.asarray(xg["lx"]).astype(np.int32)
 
     logging.info(f"write_grid: {grid_fn}")
     with h5py.File(grid_fn, "w") as h:
-        for i in (1, 2, 3):
-            for k in (
+        for i in {1, 2, 3}:
+            for k in {
                 f"x{i}",
                 f"x{i}i",
                 f"dx{i}b",
@@ -176,9 +121,7 @@ def grid(size_fn: Path, grid_fn: Path, xg: dict[str, T.Any]):
                 f"h{i}x1i",
                 f"h{i}x2i",
                 f"h{i}x3i",
-                f"gx{i}",
-                f"e{i}",
-            ):
+            }:
                 if k not in xg:
                     logging.info(f"SKIP: {k}")
                     continue
@@ -189,46 +132,82 @@ def grid(size_fn: Path, grid_fn: Path, xg: dict[str, T.Any]):
                         data=xg[k].transpose(),
                         dtype=np.float32,
                         compression="gzip",
-                        compression_opts=1,
+                        compression_opts=CLVL,
                         shuffle=True,
                         fletcher32=True,
                     )
                 else:
                     h[f"/{k}"] = xg[k].astype(np.float32)
 
-        for k in (
+        # 3-D same as grid
+        for k in {
+            "gx1",
+            "gx2",
+            "gx3",
             "alt",
             "glat",
             "glon",
             "Bmag",
-            "I",
             "nullpts",
-            "er",
-            "etheta",
-            "ephi",
             "r",
             "theta",
             "phi",
             "x",
             "y",
             "z",
-        ):
+        }:
             if k not in xg:
                 logging.info(f"SKIP: {k}")
                 continue
 
-            if xg[k].ndim >= 2:
-                h.create_dataset(
-                    f"/{k}",
-                    data=xg[k].transpose(),
-                    dtype=np.float32,
-                    compression="gzip",
-                    compression_opts=1,
-                    shuffle=True,
-                    fletcher32=True,
-                )
-            else:
-                h[f"/{k}"] = xg[k].astype(np.float32)
+            h.create_dataset(
+                f"/{k}",
+                shape=xg["lx"][::-1],
+                data=xg[k].transpose(),
+                dtype=np.float32,
+                compression="gzip",
+                compression_opts=CLVL,
+                shuffle=True,
+                fletcher32=True,
+            )
+
+        # %% 2-D
+        for k in {"I"}:
+            if k not in xg:
+                logging.info(f"SKIP: {k}")
+                continue
+
+            h.create_dataset(
+                f"/{k}",
+                shape=(xg["lx"][1], xg["lx"][2])[::-1],
+                data=xg[k].transpose(),
+                dtype=np.float32,
+                compression="gzip",
+                compression_opts=CLVL,
+                shuffle=True,
+                fletcher32=True,
+            )
+
+        # %% 4-D
+        for k in {"e1", "e2", "e3", "er", "etheta", "ephi"}:
+            if k not in xg:
+                logging.info(f"SKIP: {k}")
+                continue
+
+            h.create_dataset(
+                f"/{k}",
+                shape=(*xg["lx"], 3)[::-1],
+                data=xg[k].transpose(),
+                dtype=np.float32,
+                compression="gzip",
+                compression_opts=CLVL,
+                shuffle=True,
+                fletcher32=True,
+            )
+
+        if "glonctr" in xg:
+            h["/glonctr"] = xg["glonctr"]
+            h["/glatctr"] = xg["glatctr"]
 
 
 def Efield(outdir: Path, E: xarray.Dataset):
@@ -236,12 +215,9 @@ def Efield(outdir: Path, E: xarray.Dataset):
     write Efield to disk
     """
 
-    if h5py is None:
-        raise ImportError("pip install h5py")
-
     with h5py.File(outdir / "simsize.h5", "w") as f:
-        f["/llon"] = E.mlon.size
-        f["/llat"] = E.mlat.size
+        f.create_dataset("/llon", data=E.mlon.size, dtype=np.int32)
+        f.create_dataset("/llat", data=E.mlat.size, dtype=np.int32)
 
     with h5py.File(outdir / "simgrid.h5", "w") as f:
         f["/mlon"] = E.mlon.astype(np.float32)
@@ -249,15 +225,12 @@ def Efield(outdir: Path, E: xarray.Dataset):
 
     for t in E.time:
         time: datetime = to_datetime(t)
-        fn = outdir / (datetime2ymd_hourdec(time) + ".h5")
+        fn = outdir / (datetime2stem(time) + ".h5")
 
         # FOR EACH FRAME WRITE A BC TYPE AND THEN OUTPUT BACKGROUND AND BCs
         with h5py.File(fn, "w") as f:
             f["/flagdirich"] = E["flagdirich"].loc[time].astype(np.int32)
-            f["/time/ymd"] = [time.year, time.month, time.day]
-            f["/time/UTsec"] = (
-                time.hour * 3600 + time.minute * 60 + time.second + time.microsecond / 1e6
-            )
+            write_time(f, time)
 
             for k in {"Exit", "Eyit", "Vminx1it", "Vmaxx1it"}:
                 f.create_dataset(
@@ -265,7 +238,7 @@ def Efield(outdir: Path, E: xarray.Dataset):
                     data=E[k].loc[time].transpose(),
                     dtype=np.float32,
                     compression="gzip",
-                    compression_opts=1,
+                    compression_opts=CLVL,
                     shuffle=True,
                     fletcher32=True,
                 )
@@ -275,12 +248,9 @@ def Efield(outdir: Path, E: xarray.Dataset):
 
 def precip(outdir: Path, P: xarray.Dataset):
 
-    if h5py is None:
-        raise ImportError("pip install h5py")
-
     with h5py.File(outdir / "simsize.h5", "w") as f:
-        f["/llon"] = P.mlon.size
-        f["/llat"] = P.mlat.size
+        f.create_dataset("/llon", data=P.mlon.size, dtype=np.int32)
+        f.create_dataset("/llat", data=P.mlat.size, dtype=np.int32)
 
     with h5py.File(outdir / "simgrid.h5", "w") as f:
         f["/mlon"] = P.mlon.astype(np.float32)
@@ -288,16 +258,46 @@ def precip(outdir: Path, P: xarray.Dataset):
 
     for t in P.time:
         time: datetime = to_datetime(t)
-        fn = outdir / (datetime2ymd_hourdec(time) + ".h5")
+        fn = outdir / (datetime2stem(time) + ".h5")
 
         with h5py.File(fn, "w") as f:
+            write_time(f, to_datetime(time))
+
             for k in {"Q", "E0"}:
                 f.create_dataset(
                     f"/{k}p",
                     data=P[k].loc[time].transpose(),
                     dtype=np.float32,
                     compression="gzip",
-                    compression_opts=1,
+                    compression_opts=CLVL,
                     shuffle=True,
                     fletcher32=True,
                 )
+
+
+def maggrid(fn: Path, mag: dict[str, T.Any], gridsize: tuple[int, int, int]):
+    """
+    hdf5 files can optionally store a gridsize variable which tells readers how to
+    reshape the data into 2D or 3D arrays.
+    NOTE: the Fortran magcalc.f90 is looking for flat list.
+    """
+
+    print("write", fn)
+
+    freal = np.float32
+
+    with h5py.File(fn, "w") as f:
+        f.create_dataset("/lpoints", data=mag["r"].size, dtype=np.int32)
+        f["/r"] = mag["r"].ravel(order="F").astype(freal)
+        f["/theta"] = mag["theta"].ravel(order="F").astype(freal)
+        f["/phi"] = mag["phi"].ravel(order="F").astype(freal)
+        f["/gridsize"] = np.asarray(gridsize).astype(np.int32)
+
+
+def write_time(fid: h5py.File, time: datetime):
+    """
+    write time to HDF5 file as year-month-day, UTsec
+    """
+
+    fid["/time/ymd"] = np.array((time.year, time.month, time.day)).astype(np.int32)
+    fid["/time/UTsec"] = time.hour * 3600 + time.minute * 60 + time.second + time.microsecond / 1e6
