@@ -50,7 +50,7 @@ def hpc_batch_create(batcher: str, out_dir: Path, cmd: list[str]) -> Path:
     return job_file
 
 
-def hpc_batch_detect() -> str:
+def hpc_batch_detect() -> str | None:
     """
     Assuming a known job batching system, we will create a template for the user
     to verify and then the user will run.
