@@ -1,8 +1,6 @@
 """
-functions related to running gemini.bin.
+functions related to running Gemini3D.
 Either locally (laptop or interactive HPC) or creating an HPC batch script based on template.
-
-Builds gemini.bin if not found.
 """
 
 from __future__ import annotations
@@ -11,21 +9,22 @@ import os
 import logging
 import subprocess
 import shutil
-from pathlib import Path
+from pathlib import Path, PurePosixPath
 import numpy as np
-import psutil
 
 from . import find
-from . import mpi
-from . import cmake
 from .hpc import hpc_batch_detect, hpc_batch_create
 from . import model
 from . import write
 from . import read
+from . import wsl
 from .utils import git_meta
 
-
-Pathlike = T.Union[str, Path]
+try:
+    from math import prod
+except ImportError:
+    # python < 3.8
+    from numpy import prod  # type: ignore
 
 
 def runner(pr: dict[str, T.Any]) -> None:
@@ -35,17 +34,16 @@ def runner(pr: dict[str, T.Any]) -> None:
     config_file = find.config(pr["config_file"])
     # load configuration to know what directories to check
     p = read.config(config_file)
-    if not p:
-        raise FileNotFoundError(f"{config_file} does not appear to contain config.nml")
 
     # we don't want to overwrite an expensive simulation output
-    if find.frame(out_dir, p["time"][0], p.get("out_format")):
-        raise FileExistsError(
-            f"a fresh simulation should not have data in output directory: {out_dir}"
-        )
+    try:
+        find.frame(out_dir, p["time"][0])
+        raise FileExistsError(f"new simulation shouldn't have data in output directory: {out_dir}")
+    except FileNotFoundError:
+        pass
 
     # %% setup grid and/or initial ionosphere state if needed
-    for k in ("indat_size", "indat_grid", "indat_file"):
+    for k in {"indat_size", "indat_grid", "indat_file"}:
         f = out_dir / p[k].expanduser()
         if pr.get("force") or not f.is_file():
             model.setup(p["nml"], out_dir)
@@ -66,21 +64,15 @@ def runner(pr: dict[str, T.Any]) -> None:
             model.setup(p["nml"], out_dir)
 
     # build checks
-    gemexe = get_gemini_exe(pr.get("gemexe"))
+    gemexe = find.gemini_exe(pr.get("gemexe", ""))
+    if not gemexe:
+        raise FileNotFoundError("gemini3d.run executable not found")
     logging.info(f"gemini executable: {gemexe}")
 
-    mpiexec = check_mpiexec(pr.get("mpiexec"), gemexe)
-    if mpiexec:
-        logging.info(f"mpiexec: {mpiexec}")
-        Nmpi = mpi.count(out_dir / p["indat_size"], pr.get("cpu_count"))
-        mpi_cmd = [mpiexec, "-n", str(Nmpi)]
+    if os.name == "nt" and isinstance(gemexe, PurePosixPath):
+        cmd = ["wsl", str(gemexe), str(wsl.win_path2wsl_path(out_dir))]
     else:
-        mpi_cmd = []
-
-    cmd = mpi_cmd + [str(gemexe), str(out_dir)]
-
-    if pr.get("out_format"):
-        cmd += ["-out_format", pr["out_format"]]
+        cmd = [str(gemexe), str(out_dir)]
 
     # %% attempt dry run, but don't fail in case intended for HPC
     logging.info("Gemini dry run command:")
@@ -88,18 +80,22 @@ def runner(pr: dict[str, T.Any]) -> None:
     proc = subprocess.run(cmd + ["-dryrun"])
 
     if proc.returncode != 0:
-        raise RuntimeError("Gemini dry run failed.")
+        raise RuntimeError(f"Gemini dry run failed. {' '.join(cmd)}")
 
     if pr.get("dryrun"):
         return None
 
-    write.meta(out_dir / "setup_meta.nml", git_meta(gemexe.parent), "setup_gemini")
+    write.meta(out_dir / "setup_run.json", git_meta(gemexe.parent), p)
 
     batcher = hpc_batch_detect()
     if batcher:
         job_file = hpc_batch_create(batcher, out_dir, cmd)
         print("Please examine batch file", job_file, "and when ready submit the job as usual.")
-    else:
+        return None
+
+    try:
+        import psutil
+
         avail_memory = psutil.virtual_memory().available
         if avail_memory < 2 * ram_use_bytes:
             logging.warning(
@@ -108,20 +104,23 @@ Computer RAM available: {avail_memory/1e9:.1} GB but simulation needs {ram_use_b
 Gemini3D may run out of RAM on this computer, which may make the run exceedingly slow or fail.
 """
             )
-        print("\nBEGIN Gemini run with command:")
-        print(" ".join(cmd), "\n")
-        ret = subprocess.run(cmd).returncode
-        if ret != 0:
-            raise RuntimeError("Gemini run failed")
+    except ImportError:
+        pass
+
+    print("\nBEGIN Gemini run with command:")
+    print(" ".join(cmd), "\n")
+    ret = subprocess.run(cmd).returncode
+    if ret != 0:
+        raise RuntimeError("Gemini run failed")
 
 
 def memory_estimate(path: Path) -> int:
     """
-    Estimate how must RAM gemini.bin will need.
+    Estimate how must RAM Gemini3D will need.
     The current Gemini MPI architecture assumes the root node will use the most RAM,
     so that is the fundamental constraint.
     This neglects size of executable and library footprint,
-    which would be miniscule in simulations larger than 1 GB where we
+    which would be minuscule in simulations larger than 1 GB where we
     are concerned about RAM limits.
 
     Parameters
@@ -146,9 +145,10 @@ def memory_estimate(path: Path) -> int:
     grid_size = 0
 
     for k, v in gs.items():
-        if k == "lxs" or not isinstance(v, (tuple, list, np.ndarray)):
+        if k == "lx" or not isinstance(v, (tuple, list, np.ndarray)) or not v:
             continue
-        grid_size += np.prod(v)  # type: ignore
+        print(k, v, grid_size)
+        grid_size += int(prod(v))
 
     LSP = 7
     x1 = gs["x1"][0]
@@ -171,61 +171,47 @@ def check_compiler():
         raise EnvironmentError("Cannot find Fortran compiler e.g. Gfortran")
 
 
-def check_mpiexec(mpiexec: Pathlike, gemexe: Pathlike) -> str:
-    """check if specified mpiexec exists on this system
-    If not, fall back to not using MPI (slow, but still works)."""
+def check_mpiexec(mpiexec: str, gemexe: Path) -> str:
+    """
+    check if specified mpiexec exists on this system.
+    If not, error as most runs are exceedingly slow with one CPU core.
+    """
 
     if not mpiexec:
         mpiexec = "mpiexec"
-    mpi_root = os.environ.get("MPI_ROOT")
+
+    mpi_root = os.environ.get("MPI_ROOT", None)
     if mpi_root:
         mpi_root += "/bin"
 
-    mpiexec = shutil.which(mpiexec, path=str(mpi_root))
+    mpiexec = shutil.which(mpiexec, path=mpi_root)
     if not mpiexec:
-        return None
+        raise FileNotFoundError(f"Cannot find mpiexec {mpiexec}")
 
-    ret = subprocess.run([mpiexec, "-help"], stdout=subprocess.PIPE, text=True, timeout=5)
+    ret = subprocess.run([mpiexec, "-help"], capture_output=True, text=True, timeout=5)
     if ret.returncode != 0:
-        return None
+        raise RuntimeError(f"MPIexec error code {ret.returncode}\n{ret.stderr}")
     # %% check that compiler and MPIexec compatible
     if os.name != "nt":
         return mpiexec
 
     mpi_msg = ret.stdout.strip()
-    ret = subprocess.run([str(gemexe), "-compiler"], stdout=subprocess.PIPE, text=True, timeout=5)
+    ret = subprocess.run(
+        [str(gemexe), "-compiler"],
+        capture_output=True,
+        text=True,
+        timeout=5,
+    )
     if ret.returncode != 0:
-        raise EnvironmentError(f"{gemexe} not executable")
+        raise RuntimeError(f"{gemexe} not executable")
 
     if "GNU" in ret.stdout.strip() and "Intel(R) MPI Library" in mpi_msg:
-        mpiexec = None
-        logging.error("Not using MPIexec since MinGW is not compatible with Intel MPI")
+        raise EnvironmentError("MPIexec from MinGW is not compatible with Intel MPI")
 
     return mpiexec
 
 
-def get_gemini_exe(gemexe: Path = None) -> Path:
-    """
-    find and check that Gemini exectuable can run on this system
-    download and build Gemini3D if needed
-    """
-
-    if not gemexe:  # allow for default dict empty
-        gemexe = Path("gemini.bin")
-    gemexe = cmake.build_gemini3d(gemexe)
-
-    # %% ensure gemini.bin is runnable
-    ret = subprocess.run([str(gemexe)], stdout=subprocess.DEVNULL, timeout=15)
-    if ret.returncode != 0:
-        raise RuntimeError(
-            f"\n{gemexe} was not runnable on your platform. Try recompiling on this computer type."
-            "E.g. different HPC nodes may not have the CPU feature sets."
-        )
-
-    return gemexe
-
-
-def check_outdir(out_dir: Pathlike) -> Path:
+def check_outdir(out_dir: str | Path) -> Path:
 
     out_dir = Path(out_dir).expanduser().resolve()
     if out_dir.is_file():
