@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """
 get Linux system info
 """
@@ -36,7 +35,7 @@ def os_release() -> list[str]:
 
 
 def parse_os_release(txt: str) -> list[str]:
-    """ parse /etc/os-release text """
+    """parse /etc/os-release text"""
 
     C = ConfigParser(inline_comment_prefixes=("#", ";"))
     C.read_string(txt)
@@ -48,15 +47,17 @@ def parse_os_release(txt: str) -> list[str]:
     return like
 
 
-def get_package_manager(like: list[str] = None) -> str:
+def get_package_manager(like: list[str] | None = None) -> str:
     if not like:
         like = os_release()
     if isinstance(like, str):
         like = [like]
 
-    if {"centos", "rhel", "fedora"}.intersection(like):
+    sl = set(like)
+
+    if {"centos", "rhel", "fedora"} & sl:
         return "yum"
-    elif {"debian", "ubuntu"}.intersection(like):
+    elif {"debian", "ubuntu"} & sl:
         return "apt"
     elif like == "arch":
         return "pacman"
