@@ -4,17 +4,17 @@ setup a new simulation
 
 from __future__ import annotations
 import logging
-import importlib
 import argparse
 from pathlib import Path
 import typing as T
 import shutil
 
 from .config import read_nml
-from . import grid
+from .grid import cartesian, tilted_dipole
 from .plasma import equilibrium_state, equilibrium_resample
 from .efield import Efield_BCs
 from .particles import particles_BCs
+from .utils import str2func
 from . import namelist
 from . import write
 
@@ -45,8 +45,6 @@ def config(params: dict[str, T.Any], out_dir: Path):
 
     logging.info(f"Creating Gemini3D configuration file under {nml_file}")
 
-    file_format = params.get("file_format", "h5")
-
     # %% base
     t0 = params["time"][0]
     tend = params["time"][-1]
@@ -68,9 +66,9 @@ def config(params: dict[str, T.Any], out_dir: Path):
 
     # %% files
     files = {
-        "indat_size": f"inputs/simsize.{file_format}",
-        "indat_grid": f"inputs/simgrid.{file_format}",
-        "indat_file": f"inputs/initial_conditions.{file_format}",
+        "indat_size": "inputs/simsize.h5",
+        "indat_grid": "inputs/simgrid.h5",
+        "indat_file": "inputs/initial_conditions.h5",
     }
     namelist.write(nml_file, "files", files)
 
@@ -96,7 +94,7 @@ def config(params: dict[str, T.Any], out_dir: Path):
     namelist.write(nml_file, "setup", setup)
 
 
-def setup(path: Path | dict[str, T.Any], out_dir: Path):
+def setup(path: Path | dict[str, T.Any], out_dir: Path, root: Path | None = None):
     """
     top-level function to create a new simulation FROM A FILE config.nml
 
@@ -122,11 +120,14 @@ def setup(path: Path | dict[str, T.Any], out_dir: Path):
 
     cfg["out_dir"] = Path(out_dir).expanduser().resolve()
 
-    for k in ("indat_size", "indat_grid", "indat_file"):
+    if root:
+        cfg["gemini_root"] = Path(root).expanduser().resolve(strict=True)
+
+    for k in {"indat_size", "indat_grid", "indat_file"}:
         cfg[k] = cfg["out_dir"] / cfg[k]
 
     # FIXME: should use is_absolute() ?
-    for k in ("eq_dir", "eqzip", "E0dir", "precdir"):
+    for k in {"eq_dir", "eq_archive", "E0dir", "precdir"}:
         if cfg.get(k):
             cfg[k] = (cfg["out_dir"] / cfg[k]).resolve()
 
@@ -142,22 +143,32 @@ def setup(path: Path | dict[str, T.Any], out_dir: Path):
         equilibrium(cfg)
 
 
-def equilibrium(p: dict[str, T.Any]):
+def equilibrium(cfg: dict[str, T.Any]):
     # %% GRID GENERATION
 
-    xg = grid.cart3d(p)
+    if "lxp" in cfg and "lyp" in cfg:
+        xg = cartesian.cart3d(cfg)
+    elif "lq" in cfg and "lp" in cfg and "lphi" in cfg:
+        xg = tilted_dipole.tilted_dipole3d(cfg)
+    else:
+        raise ValueError("grid does not seem to be cartesian or curvilinear")
 
-    write.grid(p, xg)
+    write.grid(cfg, xg)
 
     # %% Equilibrium input generation
-    dat = equilibrium_state(p, xg)
+    dat = equilibrium_state(cfg, xg)
 
-    write.state(p["indat_file"], dat)
+    write.state(cfg["indat_file"], dat)
 
 
 def interp(cfg: dict[str, T.Any]) -> None:
 
-    xg = grid.cart3d(cfg)
+    if "lxp" in cfg and "lyp" in cfg:
+        xg = cartesian.cart3d(cfg)
+    elif "lq" in cfg and "lp" in cfg and "lphi" in cfg:
+        xg = tilted_dipole.tilted_dipole3d(cfg)
+    else:
+        raise ValueError("grid does not seem to be cartesian or curvilinear")
 
     equilibrium_resample(cfg, xg)
 
@@ -181,13 +192,8 @@ def postprocess(cfg: dict[str, T.Any], xg: dict[str, T.Any]) -> None:
         )
 
         for name in funcs:
-            mod_name = ".".join(name.split(".")[:-1])
-            func_name = name.split(".")[-1]
-            if not mod_name:
-                # file with single function of same name e.g. perturb.py with function perturb()
-                mod_name = func_name
-            mod = importlib.import_module(mod_name)
-            getattr(mod, func_name)(cfg, xg)
+            func = str2func(name)
+            func(cfg, xg)
 
         return
 
@@ -204,9 +210,10 @@ def cli():
     p = argparse.ArgumentParser()
     p.add_argument("config_file", help="path to config*.nml file")
     p.add_argument("out_dir", help="simulation output directory")
+    p.add_argument("--root", help="top-level path to Gemini3D installation")
     P = p.parse_args()
 
-    setup(P.config_file, P.out_dir)
+    setup(P.config_file, P.out_dir, P.root)
 
 
 if __name__ == "__main__":
