@@ -7,6 +7,7 @@ from __future__ import annotations
 import typing as T
 from pathlib import Path
 import re
+
 import numpy as np
 
 
@@ -36,7 +37,7 @@ def read(file: Path, namelist: str) -> dict[str, T.Any]:
     end_pat = re.compile(r"^\s*/\s*$")
     val_pat = re.compile(r"^\s*(\w+)\s*=\s*([^!]*)")
 
-    with file.open("r") as f:
+    with file.open("rt") as f:
         for line in f:
             if not nml_pat.match(line):
                 continue
