@@ -4,8 +4,9 @@ plasma functions
 
 from __future__ import annotations
 import typing as T
-import numpy as np
 import logging
+
+import numpy as np
 import xarray
 from scipy.integrate import cumtrapz
 from scipy.interpolate import interp1d, interp2d, interpn
@@ -13,7 +14,8 @@ from scipy.interpolate import interp1d, interp2d, interpn
 from . import read
 from . import LSP, SPECIES
 from . import write
-from .web import url_retrieve, extract_zip
+from .web import url_retrieve
+from .archive import extract
 from .msis import msis_setup
 
 # CONSTANTS
@@ -33,15 +35,11 @@ def equilibrium_resample(p: dict[str, T.Any], xg: dict[str, T.Any]):
             raise FileNotFoundError(
                 f"{p['eq_dir']} not found and eq_url not specified in {p['nml']}"
             )
-        url_retrieve(p["eq_url"], p["eq_zip"])
-        extract_zip(p["eq_zip"], p["eq_dir"])
+        url_retrieve(p["eq_url"], p["eq_archive"])
+        extract(p["eq_archive"], p["eq_dir"])
 
     # %% READ Equilibrium SIMULATION INFO
     peq = read.config(p["eq_dir"])
-    if not peq:
-        raise FileNotFoundError(
-            f"equilibrium directory {p['eq_dir']} does not appear to contain config.nml"
-        )
 
     # %% END FRAME time of equilibrium simulation
     # this will be the starting time of the new simulation
@@ -59,8 +57,6 @@ def equilibrium_resample(p: dict[str, T.Any], xg: dict[str, T.Any]):
 
     # %% DO THE INTERPOLATION
     xg_in = read.grid(p["eq_dir"])
-    if not xg_in:
-        raise FileNotFoundError(f"{p['eq_dir']} does not have an input simulation grid.")
 
     dat_interp = model_resample(xg_in, dat, xg)
 
@@ -72,7 +68,7 @@ def equilibrium_resample(p: dict[str, T.Any], xg: dict[str, T.Any]):
     # %% WRITE OUT THE GRID
     write.grid(p, xg)
 
-    write.state(p["indat_file"], dat_interp, file_format=p.get("file_format"))
+    write.state(p["indat_file"], dat_interp)
 
 
 def model_resample(
@@ -135,10 +131,10 @@ def model_resample(
 
         for i in range(LSP):
             for k in {"ns", "vs1", "Ts"}:
-                # the .values are to avoid OutOfMemoryError
+                # the .data is to avoid OutOfMemoryError
                 dat_interp[k][i, :, :, :] = interpn(
                     points=(X1, X2, X3),
-                    values=dat[k][i, :, :, :].values,
+                    values=dat[k][i, :, :, :].data,
                     xi=(X1i, X2i, X3i),
                     bounds_error=True,
                 )
@@ -188,7 +184,7 @@ def model_resample(
     return dat_interp
 
 
-def check_density(n: np.ndarray):
+def check_density(n: xarray.DataArray):
 
     if not np.isfinite(n).all():
         raise ValueError("non-finite density")
@@ -198,7 +194,7 @@ def check_density(n: np.ndarray):
         raise ValueError("too small maximum density")
 
 
-def check_drift(v: np.ndarray):
+def check_drift(v: xarray.DataArray):
 
     if not np.isfinite(v).all():
         raise ValueError("non-finite drift")
@@ -206,7 +202,7 @@ def check_drift(v: np.ndarray):
         raise ValueError("excessive drift velocity")
 
 
-def check_temperature(Ts: np.ndarray):
+def check_temperature(Ts: xarray.DataArray):
 
     if not np.isfinite(Ts).all():
         raise ValueError("non-finite temperature")
@@ -227,12 +223,12 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
     # %% MAKE UP SOME INITIAL CONDITIONS FOR FORTRAN CODE
     mindens = 1e-100
 
-    def Oplus(ns: np.ndarray) -> np.ndarray:
+    def Oplus(ns):
         ns[0, :, ix2, ix3] = rho * ne
         zref = 900e3
         i = alt[:, ix2, ix3] > zref
         if any(i):
-            iord = np.argsort(alt[:, ix2, ix3])
+            iord = alt[:, ix2, ix3].argsort()
             altsort = alt[iord, ix2, ix3]
             nsort = ns[0, :, ix2, ix3]
             nsort = nsort[iord]
@@ -241,7 +237,7 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
             H = KB * 2 * Tn[inds, ix2, ix3] / ms / g[inds, ix2, ix3]
             z = alt[i, ix2, ix3]
             lz = z.size
-            iord = np.argsort(z)
+            iord = z.argsort()
             z = z[iord]
             #     z=[z; 2*z(lz)-z(lz-1)];
             z = np.insert(z, 0, zref)
@@ -260,7 +256,7 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
 
         return ns
 
-    def molecular_density(ns: np.ndarray, xgr: np.ndarray, inds: np.ndarray) -> np.ndarray:
+    def molecular_density(ns, xgr, inds):
         """MOLECULAR DENSITIES
 
         Parameters
@@ -286,8 +282,6 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
         nmolc = np.zeros(lx1)
         nmolc[i] = (1 - rho[i]) * ne[i]
 
-        cond: bool = None
-
         if any(inds):
             if xgr.ndim == 3:
                 cond = xgr[0, 0, 0] > xgr[1, 0, 0]
@@ -305,9 +299,10 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
             H = KB * Tn[inds, ix2, ix3] / ms / g[inds, ix2, ix3]
             z = alt[inds, ix2, ix3]
             lz = z.size
-            iord = np.argsort(z)
+            iord = z.argsort()
             z = z[iord]
-            z = np.append(z, 2 * z[-1] - z[-2])
+            # z = np.append(z, 2 * z[-1] - z[-2])
+            z = np.insert(z, 0, alt[iref, ix2, ix3])
             integrand = 1 / H[iord]
             integrand = np.append(integrand, integrand[-1])
             # this cumtrapz() does NOT get initial=0, since matlab user code strips first element here
@@ -340,26 +335,25 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
     # %% SLICE THE FIELD IN HALF IF WE ARE CLOSED
     atmos = msis_setup(p, xg)
 
-    closeddip = abs(xg["r"][0, 0, 0] - xg["r"][-1, 0, 0]) < 50e3
+    closeddip: bool = abs(xg["r"][0, 0, 0] - xg["r"][-1, 0, 0]) < 50e3
     # logical flag marking the grid as closed dipole
     if closeddip:
         # closed dipole grid
-        #    [~,ialtmax]=max(xg.alt(:,1,1))
-        #    lalt=ialtmax
-        lalt = xg["lx"][0] // 2
+        i = xg["alt"][:, 0, 0].argmax()
+        # i = xg["lx"][0] // 2
         # FIXME:  needs to work with asymmetric grid...
-        alt = xg["alt"][:lalt, :, :]
-        lx1 = lalt
+        alt = xg["alt"][:i, :, :]
+        lx1 = i
         lx2 = xg["lx"][1]
         lx3 = xg["lx"][2]
-        Tn = atmos["Tn"][:lalt, :, :]
-        g = abs(xg["gx1"][:lalt, :, :])
-        g = max(g, 1)
+        Tn = atmos["Tn"][:i, :, :]
+        g = abs(xg["gx1"][:i, :, :])
+        g = g.clip(min=1)
         for ix3 in range(lx3):
             for ix2 in range(lx2):
                 ialt = abs(g[:, ix2, ix3] - 1).argmin()
-                if ialt != lalt:
-                    g[ialt:lalt, ix2, ix3] = 1
+                if ialt != i:
+                    g[ialt:i, ix2, ix3] = 1
 
     else:
         alt = xg["alt"]
@@ -374,6 +368,7 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
             z0f = 325e3
             He = 2 * KB * Tn[:, ix2, ix3] / AMU / 30 / g[:, ix2, ix3]
             z0e = 120e3
+
             ne = chapmana(alt[:, ix2, ix3], p["nmf"], z0f, Hf) + chapmana(
                 alt[:, ix2, ix3], p["nme"], z0e, He
             )
@@ -388,7 +383,7 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
                 H = KB * 2 * Tn[inds, ix2, ix3] / ms / g[inds, ix2, ix3]
                 z = alt[inds, ix2, ix3]
                 lz = z.size
-                iord = np.argsort(z)
+                iord = z.argsort()
                 z = z[iord]
                 #     z=[z; 2*z(lz)-z(lz-1)];
                 z = np.insert(z, 0, z0f)
@@ -420,14 +415,27 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
     if closeddip:
         # closed dipole grid
         # FIXME:  This code only works for symmetric grids...
+
         if 2 * lx1 == xg["lx"][0]:
             ns = np.concatenate((ns, ns[:, ::-1, :, :]), 1)
             Ts = np.concatenate((Ts, Ts[:, ::-1, :, :]), 1)
             vsx1 = np.concatenate((vsx1, vsx1[:, ::-1, :, :]), 1)
         else:
-            ns = np.concatenate((ns, ns[:, lx1, :, :], ns[:, ::-1, :, :]), 1)
-            Ts = np.concatenate((Ts, Ts[:, lx1, :, :], Ts[:, ::-1, :, :]), 1)
-            vsx1 = np.concatenate((vsx1, vsx1[:, lx1, :, :], vsx1[:, ::-1, :, :]), 1)
+            nsp = ns[:, -1, :, :][:, None, :, :]
+            tsp = Ts[:, -1, :, :][:, None, :, :]
+            vsp = vsx1[:, -1, :, :][:, None, :, :]
+            if 2 * lx1 == xg["lx"][0] - 1:
+                # off by one, fill with end point
+                ns = np.concatenate((ns, nsp, ns[:, ::-1, :, :]), 1)
+                Ts = np.concatenate((Ts, tsp, Ts[:, ::-1, :, :]), 1)
+                vsx1 = np.concatenate((vsx1, vsp, vsx1[:, ::-1, :, :]), 1)
+            elif 2 * lx1 == xg["lx"][0] - 2:
+                # off by two, fill with end point
+                ns = np.concatenate((ns, nsp, nsp, ns[:, ::-1, :, :]), 1)
+                Ts = np.concatenate((Ts, tsp, tsp, Ts[:, ::-1, :, :]), 1)
+                vsx1 = np.concatenate((vsx1, vsp, vsp, vsx1[:, ::-1, :, :]), 1)
+            else:
+                raise ValueError(f"closed dipole lalt: 2*lx1={2*lx1} != grid lx1: {xg['lx'][0]}")
 
     dat = xarray.Dataset(
         {
@@ -447,7 +455,29 @@ def equilibrium_state(p: dict[str, T.Any], xg: dict[str, T.Any]) -> xarray.Datas
     return dat
 
 
-def chapmana(z: np.ndarray, nm: float, z0: float, H: float) -> np.ndarray:
+def chapmana(z, nm: float, z0: float, H):
+    """
+    create Chapman profile
+
+    Parameters
+    ----------
+
+    z : np.ndarray
+        altitude [meters]
+
+    nm : float
+        number density scale factor
+
+    z0 : float
+        peak altitude [meters]
+
+    H : float or np.ndarray
+        scale height vs. altitude z [meters]
+    """
+
+    if np.any(H < 0.1):
+        raise ValueError(f"scale height H is unrealistically small:  {H}")
+
     zref = (z - z0) / H
     ne = nm * np.exp(0.5 * (1 - zref - np.exp(-zref)))
 
