@@ -1,24 +1,70 @@
 from __future__ import annotations
 import subprocess
 import os
-import re
 import shutil
 from pathlib import Path
-import xarray
-import numpy as np
-from datetime import datetime
+
+from datetime import datetime, timedelta
 import typing as T
 import logging
+import importlib
+import importlib.util
 
-try:
-    import psutil
-except ImportError:
-    psutil = None
-    # pip install psutil will improve CPU utilization.
+# the importlib.util import is necessary on non-Windows systems at least for MyPy 0.971
 
-Pathlike = T.Union[str, Path]
+import xarray
+import numpy as np
 
-__all__ = ["to_datetime", "git_meta", "get_cpu_count", "datetime2ymd_hourdec"]
+
+__all__ = ["to_datetime", "git_meta", "datetime2stem"]
+
+
+def str2func(name: str, path: Path | None = None) -> T.Callable:
+    """
+    expects one of (in priority order):
+
+    0. file in "path" (if present)
+    1. os.getcwd()/name.py containing function name()
+    2. gemiin3d.<foo> <foo>/name.py module file containing function name()
+    3. gemini3d.<foo> <foo>/__init__.py containing function name()
+
+
+    Examples:
+
+    1. os.getcwd()/perturb.py with function perturb()
+    2. gemini3d.efield.Efield_erf returns function Efield_erf()
+    """
+
+    mod_name = ".".join(name.split(".")[:-1])
+    func_name = name.split(".")[-1]
+
+    if mod_name:
+        try:
+            # file with function of same name
+            mod = importlib.import_module(name)
+            return getattr(mod, func_name)
+        except (ModuleNotFoundError, AttributeError):
+            # __init__.py with function
+            mod = importlib.import_module(mod_name)
+            return getattr(mod, func_name)
+    else:
+        if path is None:
+            # file in current working directory
+            mod = importlib.import_module(func_name)
+        else:
+            # https://docs.python.org/3.10/library/importlib.html#importing-a-source-file-directly
+            mod_file = path / (name + ".py")
+            if not mod_file.is_file():
+                raise FileNotFoundError(mod_file)
+            spec = importlib.util.spec_from_file_location(name, mod_file)
+            if spec is None:
+                raise ModuleNotFoundError(f"{name} not found in {mod_file}")
+            mod = importlib.util.module_from_spec(spec)
+            if mod is None:
+                raise ImportError(f"could not import {name} from {mod_file}")
+            spec.loader.exec_module(mod)  # type: ignore
+
+    return getattr(mod, func_name)
 
 
 def to_datetime(times: xarray.DataArray | np.datetime64 | datetime) -> datetime:
@@ -36,19 +82,19 @@ def to_datetime(times: xarray.DataArray | np.datetime64 | datetime) -> datetime:
     if isinstance(times, datetime):
         time = times
     elif isinstance(times, xarray.DataArray):
-        time = times.values.squeeze()[()]  # numpy.datetime64
+        time = times.data.squeeze()[()]
     elif isinstance(times, np.datetime64):
-        time = times.squeeze()[()]
+        time = times.squeeze()[()]  # type: ignore
     else:
         raise TypeError("expected datetime-like value")
 
     if isinstance(time, np.datetime64):
-        time = time.astype("datetime64[us]").astype(datetime)
+        time = time.astype("datetime64[us]").astype(datetime)  # type: ignore
 
     return time
 
 
-def git_meta(path: Path = None) -> dict[str, str]:
+def git_meta(path: Path | None = None) -> dict[str, str]:
     """
     provide metadata about a Git repo in a dictionary
 
@@ -60,10 +106,10 @@ def git_meta(path: Path = None) -> dict[str, str]:
 
     git = shutil.which("git")
     meta = {
-        "git_version": None,
-        "remote": None,
-        "branch": None,
-        "commit": None,
+        "version": "",
+        "remote": "",
+        "branch": "",
+        "commit": "",
         "porcelain": "false",
     }
     if not git:
@@ -80,7 +126,7 @@ def git_meta(path: Path = None) -> dict[str, str]:
         logging.error("Git was not available or is too old")
         return meta
 
-    meta["git_version"] = ret.stdout.strip()
+    meta["version"] = ret.stdout.strip()
 
     ret = subprocess.run([git, "-C", str(path), "rev-parse"])
     if ret.returncode != 0:
@@ -93,23 +139,27 @@ def git_meta(path: Path = None) -> dict[str, str]:
         text=True,
     )
     if ret.returncode != 0:
-        logging.error(f"{path} coul not determine Git branch")
+        logging.error(f"{path} could not determine Git branch")
         return meta
     meta["branch"] = ret.stdout.strip()
 
-    ret = subprocess.run([git, "-C", str(path), "remote", "-v"], stdout=subprocess.PIPE, text=True)
+    ret = subprocess.run(
+        [git, "-C", str(path), "remote", "get-url", "origin"], stdout=subprocess.PIPE, text=True
+    )
     if ret.returncode != 0:
         logging.error(f"{path} could not determine Git remote")
         return meta
-
-    pat = re.compile(r"^origin\s+(.*)\s+\(fetch\)")
-    mat = pat.match(ret.stdout.strip())
-    if mat:
-        meta["remote"] = mat.group(1)
+    meta["remote"] = ret.stdout.strip()
 
     ret = subprocess.run(
-        [git, "-C", str(path), "rev-parse", "--short", "HEAD"], stdout=subprocess.PIPE, text=True
+        [git, "-C", str(path), "describe", "--tags"], stdout=subprocess.PIPE, text=True
     )
+    if ret.returncode != 0:
+        ret = subprocess.run(
+            [git, "-C", str(path), "rev-parse", "--short", "HEAD"],
+            stdout=subprocess.PIPE,
+            text=True,
+        )
     if ret.returncode != 0:
         logging.error(f"{path} could not determine Git commit")
         return meta
@@ -130,7 +180,7 @@ def get_cpu_count() -> int:
     """get a physical CPU count
 
     Note: len(os.sched_getaffinity(0)) and multiprocessing.cpu_count don't help either
-    PSUtil is the most reliable, so we strongly recommend it.
+    We'd like to use HWLOC instead, but for now we leave PSUtil.
 
     Returns
     -------
@@ -139,24 +189,24 @@ def get_cpu_count() -> int:
     """
 
     extradiv = 1
-    max_cpu = None
     # without psutil, hyperthreaded CPU may overestimate physical count by factor of 2 (or more)
-    if psutil is not None:
+    try:
+        import psutil
+
         max_cpu = psutil.cpu_count(logical=False)
         if max_cpu is None:
             max_cpu = psutil.cpu_count()
             extradiv = 2
-    if max_cpu is None:
+    except ImportError:
         max_cpu = os.cpu_count()
-        if max_cpu is not None:
-            extradiv = 2
-        else:
-            max_cpu = 1
+        extradiv = 2
 
-    return max_cpu // extradiv
+    max_cpu = 1 if max_cpu is None else max_cpu // extradiv
+
+    return max_cpu
 
 
-def datetime2ymd_hourdec(dt: datetime) -> str:
+def datetime2stem(dt: datetime) -> str:
     """
     convert datetime to ymd_hourdec string for filename stem
     """
@@ -169,3 +219,15 @@ def datetime2ymd_hourdec(dt: datetime) -> str:
         dt.strftime("%Y%m%d")
         + f"_{dt.hour*3600 + dt.minute*60 + dt.second + dt.microsecond/1e6:12.6f}"
     )
+
+
+def filename2datetime(path: Path | str) -> datetime:
+    """
+    Gemini3D datafiles use a file naming pattern that we translate to a datetime
+
+    path need not exist.
+    """
+
+    name = path.name if isinstance(path, Path) else path
+
+    return datetime.strptime(name[:8], "%Y%m%d") + timedelta(seconds=float(name[9:21]))
